@@ -20,55 +20,136 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-"""A simple colormap module.
-"""
+"""A simple colormap module."""
 
 import numpy as np
 from trollimage.colorspaces import rgb2hcl, hcl2rgb
 
 
 def colorize(arr, colors, values):
-    """Colorize a monochromatic array *arr*, based *colors* given for
-    *values*. Interpolation is used. *values* must be in ascending order.
+    """Colorize a monochromatic array *arr*, based *colors* given for *values*.
+
+    Interpolation is used. *values* must be in ascending order.
+
+    Args:
+        arr (numpy array, numpy masked array, dask array)
+            data to be colorized.
+        colors (numpy array):
+            the colors to use (R, G, B)
+        values (numpy array):
+            the values corresponding to the colors in the array
     """
-    hcolors = np.array([rgb2hcl(*i[:3]) for i in colors])
-    # unwrap colormap in hcl space
-    hcolors[:, 0] = np.rad2deg(np.unwrap(np.deg2rad(np.array(hcolors)[:, 0])))
+    if can_be_block_mapped(arr):
+        return _colorize_dask(arr, colors, values)
+    else:
+        return _colorize(arr, colors, values)
+
+
+def can_be_block_mapped(data):
+    """Check if the array can be processed in chunks."""
+    return hasattr(data, 'map_blocks')
+
+
+def _colorize_dask(dask_array, colors, values):
+    """Colorize a dask array.
+
+    The channels are stacked on the first dimension.
+    """
+    return dask_array.map_blocks(_colorize, colors, values, dtype=colors.dtype, new_axis=0,
+                                 chunks=[colors.shape[1]] + list(dask_array.chunks))
+
+
+def _colorize(arr, colors, values):
+    """Colorize the array."""
+    channels = _interpolate_rgb_colors(arr, colors, values)
+    alpha = _interpolate_alpha(arr, colors, values)
+    channels.extend(alpha)
+    channels = _mask_channels(channels, arr)
+    return np.stack(channels, axis=0)
+
+
+def _interpolate_rgb_colors(arr, colors, values):
+    hcl_colors = _convert_rgb_list_to_hcl(colors)
     channels = [np.interp(arr,
                           np.array(values),
-                          np.array(hcolors)[:, i])
+                          np.array(hcl_colors)[:, i])
                 for i in range(3)]
-
     channels = list(hcl2rgb(*channels))
+    return channels
 
-    rest = [np.interp(arr,
-                      np.array(values),
-                      np.array(colors)[:, i + 3])
-            for i in range(np.array(colors).shape[1] - 3)]
 
-    channels.extend(rest)
+def _convert_rgb_list_to_hcl(colors):
+    hcl_colors = np.array([rgb2hcl(*i[:3]) for i in colors])
+    _unwrap_colors_in_hcl_space(hcl_colors)
+    return hcl_colors
 
+
+def _unwrap_colors_in_hcl_space(hcl_colors):
+    hcl_colors[:, 0] = np.rad2deg(np.unwrap(np.deg2rad(np.array(hcl_colors)[:, 0])))
+
+
+def _interpolate_alpha(arr, colors, values):
+    alpha = [np.interp(arr,
+                       np.array(values),
+                       np.array(colors)[:, i + 3])
+             for i in range(np.array(colors).shape[1] - 3)]
+    return alpha
+
+
+def _mask_channels(channels, arr):
+    """Mask the channels if arr is a masked array."""
     try:
-        return [np.ma.array(channel, mask=arr.mask) for channel in channels]
+        channels = [_mask_array(channel, arr) for channel in channels]
     except AttributeError:
-        return channels
+        pass
+    return channels
+
+
+def _mask_array(new_array, arr):
+    """Mask new_array with the mask from array."""
+    try:
+        return np.ma.array(new_array, mask=arr.mask)
+    except AttributeError:
+        return new_array
 
 
 def palettize(arr, colors, values):
-    """From start *values* apply *colors* to *data*.
+    """Apply *colors* to *data* from start *values*.
+
+    Args:
+        arr (numpy array, numpy masked array, dask array):
+            data to be palettized.
+        colors (numpy array):
+            the colors to use (R, G, B)
+        values (numpy array):
+            the values corresponding to the colors in the array
     """
+    if can_be_block_mapped(arr):
+        return _palettize_dask(arr, colors, values), tuple(colors)
+    else:
+        return _palettize(arr, values), tuple(colors)
+
+
+def _palettize_dask(darr, colors, values):
+    """Apply a palette to a dask array."""
+    return darr.map_blocks(_palettize, values, dtype=colors.dtype)
+
+
+def _palettize(arr, values):
+    """Apply palette to array."""
+    new_arr = _digitize_array(arr, values)
+    reshaped_array = new_arr.reshape(arr.shape)
+    return _mask_array(reshaped_array, arr)
+
+
+def _digitize_array(arr, values):
     new_arr = np.digitize(arr.ravel(),
                           np.concatenate((values,
                                           [max(np.nanmax(arr),
                                                values.max()) + 1])))
     new_arr -= 1
     new_arr = new_arr.clip(min=0, max=len(values) - 1)
-    try:
-        new_arr = np.ma.array(new_arr.reshape(arr.shape), mask=arr.mask)
-    except AttributeError:
-        new_arr = new_arr.reshape(arr.shape)
-
-    return new_arr, tuple(colors)
+    return new_arr
 
 
 class Colormap(object):
@@ -89,6 +170,7 @@ class Colormap(object):
     """
 
     def __init__(self, *tuples, **kwargs):
+        """Set up the instance."""
         if 'colors' in kwargs and 'values' in kwargs:
             values = kwargs['values']
             colors = kwargs['colors']
@@ -99,31 +181,26 @@ class Colormap(object):
         self.colors = np.array(colors)
 
     def colorize(self, data):
-        """Colorize a monochromatic array *data*, based on the current colormap.
-        """
-        return colorize(data,
-                        self.colors,
-                        self.values)
+        """Colorize a monochromatic array *data*, based on the current colormap."""
+        return colorize(data, self.colors, self.values)
 
     def palettize(self, data):
-        """Palettize a monochromatic array *data* based on the current colormap.
-        """
+        """Palettize a monochromatic array *data* based on the current colormap."""
         return palettize(data, self.colors, self.values)
 
     def __add__(self, other):
+        """Append colormap together."""
         new = Colormap()
         new.values = np.concatenate((self.values, other.values))
         new.colors = np.concatenate((self.colors, other.colors))
         return new
 
     def reverse(self):
-        """Reverse the current colormap in place.
-        """
+        """Reverse the current colormap in place."""
         self.colors = np.flipud(self.colors)
 
     def set_range(self, min_val, max_val):
-        """Set the range of the colormap to [*min_val*, *max_val*]
-        """
+        """Set the range of the colormap to [*min_val*, *max_val*]."""
         if min_val > max_val:
             max_val, min_val = min_val, max_val
         self.values = (((self.values * 1.0 - self.values.min()) /
@@ -131,8 +208,7 @@ class Colormap(object):
                        * (max_val - min_val) + min_val)
 
     def to_rio(self):
-        """Converts the colormap to a rasterio colormap.
-        """
+        """Convert the colormap to a rasterio colormap."""
         self.colors = (((self.colors * 1.0 - self.colors.min()) /
                         (self.colors.max() - self.colors.min())) * 255)
         return dict(zip(self.values, tuple(map(tuple, self.colors))))
@@ -447,8 +523,7 @@ qualitative_colormaps = [set1, set2, set3,
 
 
 def colorbar(height, length, colormap):
-    """Return the channels of a colorbar.
-    """
+    """Return the channels of a colorbar."""
     cbar = np.tile(np.arange(length) * 1.0 / (length - 1), (height, 1))
     cbar = (cbar * (colormap.values.max() - colormap.values.min())
             + colormap.values.min())
@@ -457,8 +532,7 @@ def colorbar(height, length, colormap):
 
 
 def palettebar(height, length, colormap):
-    """Return the channels of a palettebar.
-    """
+    """Return the channels of a palettebar."""
     cbar = np.tile(np.arange(length) * 1.0 / (length - 1), (height, 1))
     cbar = (cbar * (colormap.values.max() + 1 - colormap.values.min())
             + colormap.values.min())
